// $Id: DefaultEventHandler.java 10479 2009-07-10 09:51:07Z chris $

package org.faceless.pdf2.viewer;

import java.awt.*;
import java.util.*;
import java.awt.geom.*;
import java.awt.event.*;
import org.faceless.pdf2.*;
import org.faceless.pdf2.Event;

/**
 * <p>
 * This class represents the default event handler used by the viewer. It handles
 * only the most basic operations - clicking on hyperlinks, mouseover and mouseout
 * actions on Widgets and so on, but can be used as a base or extended.
 * </p>
 * @since 2.7.8
 */
public class DefaultEventHandler extends EventHandler
{
    private PDFAnnotation lastannot;

    public DefaultEventHandler() {
        super(TYPE_MOUSE | TYPE_MOUSEMOTION);
    }

    public void mousePressed(PDFMouseEvent event) {
        PDFAnnotation annot = event.getAnnotation();
        if (annot!=null && annot instanceof WidgetAnnotation) {
            getViewport().runAction(((WidgetAnnotation)annot).getAction(Event.MOUSEDOWN));
        }
    }

    public void mouseReleased(PDFMouseEvent event) {
        PDFAnnotation annot = event.getAnnotation();
        if (annot!=null && annot instanceof WidgetAnnotation) {
            getViewport().runAction(((WidgetAnnotation)annot).getAction(Event.MOUSEUP));
        }
    }

    public void mouseMoved(PDFMouseEvent event) {
        PDFAnnotation annot = event.getAnnotation();

        // If the mouse has moved off or on to an annotation, run the actions and change the cursor.
        if (lastannot!=annot) {
            PDFViewport viewport = getViewport();
            viewport.setCursor(Cursor.getDefaultCursor());
            if (lastannot instanceof WidgetAnnotation) {
                viewport.runAction(((WidgetAnnotation)lastannot).getAction(Event.MOUSEOUT));
            }

            if (annot!=null) {
                viewport.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                if (annot instanceof WidgetAnnotation) {
                    viewport.runAction(((WidgetAnnotation)annot).getAction(Event.MOUSEOVER));
                }
            }
            lastannot = annot;
        }
    }

    public void mouseClicked(PDFMouseEvent event) {
        PDFAnnotation annot = event.getAnnotation();
        if (annot!=null) {
            Point2D annotpoint = event.getAnnotationPoint(annot);

            if (annot instanceof AnnotationLink) {
                getViewport().runAction(((AnnotationLink)annot).getAction());
            } else if (annot instanceof WidgetAnnotation) {
                getViewport().runAction(((WidgetAnnotation)annot).getAction(Event.MOUSEUP));
            }
        }
    }
}
